/* osgVisual test. distortionNG, experimental.
*
*  Permission is hereby granted, free of charge, to any person obtaining a copy
*  of this software and associated documentation files (the "Software"), to deal
*  in the Software without restriction, including without limitation the rights
*  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
*  copies of the Software, and to permit persons to whom the Software is
*  furnished to do so, subject to the following conditions:
*
*  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
*  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
*  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
*  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
*  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
*  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
*  THE SOFTWARE.
*/

#ifndef OSGGA_DISTORTION_MANIPULATOR
#define OSGGA_DISTORTION_MANIPULATOR 1

#include <osgGA/GUIEventHandler>
#include <osg/Referenced>
#include <osg/Switch>
#include <osg/Geometry>
#include <osgUtil/IntersectionVisitor>
#include <osgUtil/LineSegmentIntersector>
#include <osgText/Text>

#include "DistortionSetupStrategy.h"
#include "DistortionSet.h"


namespace osgViewer {


class DistortionManipulator : public osgGA::GUIEventHandler
{
public:
	enum SetupMode {
		DISABLED,
		MANUAL,
		DELEGATED
	};
	enum DistortionMode {
		MESH,
		TEXCOORDINATES
	};
	enum ManualSetupMode {
		DISTORTION,
		BLENDING
	};
	enum VisualizationMode {
		DISTORTION_MESH,
		INTENSITY_MAP,
		NONE
	};


	DistortionManipulator(DistortionSet* ds);
	virtual ~DistortionManipulator();

	virtual void getUsage(osg::ApplicationUsage& usage) const;
    virtual bool handle(const osgGA::GUIEventAdapter& ea,osgGA::GUIActionAdapter& aa, osg::Object*, osg::NodeVisitor* nv);

	void setDistortionSet(DistortionSet* ds) {_distortionSet = ds;}

	

private:
	void resetIntensityMap();
	void resetDistortion();
	void showDistortionMesh(bool show);
	void showIntensityMap(bool show);

	void computeSelectedVertex( osgUtil::LineSegmentIntersector::Intersection& result );
	void createVertexHighlighter();
	void createHUD();
	void updateHUD();

	SetupMode activeSetupMode;
	DistortionMode activeDistortionMode;
	ManualSetupMode activeManualSetupMode;
	VisualizationMode activeVisualizationMode;

	osg::ref_ptr<DistortionSet> _distortionSet;
	osg::ref_ptr<DistortionSetupStrategy> _delegatedDistortionSetupStrategy;

	osg::ref_ptr<osg::Geometry> _highlighter;
    osg::observer_ptr<osg::Camera> _camera;
	osg::Geometry* _distortionMesh;
	const osg::Vec4 _highlightColor;
	
	//LF Screen Resolution
	unsigned int _screenPixWidth, _screenPixHeight;
	

	// HUD
	osg::ref_ptr<osgText::Text> hudSetupMode;
	osg::ref_ptr<osgText::Text> hudDistortionMode;
	osg::ref_ptr<osgText::Text> hudManualSetupMode;
	osg::ref_ptr<osgText::Text> hudVisualizationMode;
};


}

#endif /* OSGGA_DISTORTION_MANIPULATOR */
